/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MyDelegate.as 9583 2012-01-11 11:48:56Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.business {
import mx.collections.ArrayCollection;
import mx.controls.Alert;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;
import org.ow2.kerneos.common.business.AbsDelegateResponder;

/**
 * @author Philippe Durieux
 */
public class NotificationEditorDelegate extends AbsDelegateResponder {

    private var service:Object = null;

    public function NotificationEditorDelegate() {
        super();
        service = NotificationEditor.getInstance().getServices().getRemoteObject("notificationEditor_ruleEngineService");
        if (service == null) {
            Alert.show("Cannot get notificationEditor_ruleEngineService", "ERROR");
        }
    }

    /**
     * Compile the rule package
     */
    public function compileRulePackage(rulePackage:IRulePackageVO, ruleEngineName:String):void {
        var call:Object = service.compileRulePackage(rulePackage, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Save the rule package
     */
    public function saveRulePackage(rulePackage:IRulePackageVO, ruleEngineName:String):void {
        var call:Object = service.saveRulePackage(rulePackage, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Check the given name availability
     */
    public function checkSystemNameAvailability(systemName:String, ruleEngineName:String):void {
        var call:Object = service.checkNameAvailability(systemName, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Export the rule packages as XML
     */
    public function exportRulePackagesAsXML(rulePackages:ArrayCollection, ruleEngineName:String):void {
        var call:Object = service.exportRulePackagesAsXML(rulePackages, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Get the rule package
     */
    public function getRulePackage(rulePackageSystemName:String, ruleEngineName:String):void {
        var call:Object = service.getRulePackage(rulePackageSystemName, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
    * Import the rule package
    */
    public function importRulePackage(filePath:String, ruleEngineName:String) : void {
        var call:Object = service.importRulePackageFromXML(filePath, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Delete the rule package
     */
    public function deleteRulePackage(rulePackage:IRulePackageVO, ruleEngineName:String):void {
        var call:Object = service.deleteRulePackage(rulePackage, ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Get the rule packages
     */
    public function getRulePackages(ruleEngineName:String):void {
        var call:Object = service.getRulePackages(ruleEngineName);
        call.addResponder(this.responder);
    }

    public function startEngine(ruleEngineName:String):void {
        var call:Object = service.start(ruleEngineName);
        call.addResponder(this.responder);
    }

    public function stopEngine(ruleEngineName:String):void {
        var call:Object = service.stop(ruleEngineName);
        call.addResponder(this.responder);
    }

    /**
     * Update the rule engine status
     */
    public function updateEngineStatus(ruleEngineName:String):void {
        var call:Object = service.getRuleEngineStatus(ruleEngineName);
        call.addResponder(this.responder);
    }


}
}
