/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CheckSystemNameAvailabilityCommand.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;

import flash.utils.getQualifiedClassName;

import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.kerneos.common.event.KerneosNotificationEvent;

/**
 * Check the rule package name availability
 *
 * @author Julien Nicoulaud
 */
public class CheckSystemNameAvailabilityCommand extends AbstractCommand {

    private var synchronizedRulePackage:SynchronizedRulePackage = null;

    public function CheckSystemNameAvailabilityCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    override public function execute(event:CairngormEvent):void {
        // Retrieve the target object
        synchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;

        // Dispatch a notification to Kerneos
        debug("Check name availability for \"" + synchronizedRulePackage.rulePackage.displayName + "\"");

        delegate.responder = this;

        // If the name is new or has changed (ie not equals to the old)
        if (synchronizedRulePackage.rulePackage.previousSystemName == null ||
                synchronizedRulePackage.rulePackage.previousSystemName !== synchronizedRulePackage.rulePackage.systemName) {

            // Execute a server-side method thanks to the delegate
            delegate.checkSystemNameAvailability(synchronizedRulePackage.rulePackage.systemName,
                    model.ruleEngineName);
        }
    }

    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    override public function result(event:Object):void {
        // Update the rule package
        synchronizedRulePackage.rulePackage.nameIsAvailable = (event as ResultEvent).result as Boolean;
    }


}
}
