/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConvertToTextualPackageCommand.as 8512 2011-07-12 07:00:31Z diorcety $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IVisualRulePackageVO;

/**
 * Convert the package to a textual one
 *
 * @author Julien Nicoulaud
 */
public class ConvertToTextualPackageCommand extends AbstractCommand  {

    public function ConvertToTextualPackageCommand() {
    }

    /**
     * Execute the command
     */
    override public function execute(event:CairngormEvent):void {

        // Get the target rule package
        var synchronizedRulePackage:SynchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;

        // Dispatch a notification to Kerneos
        debug("Converting \"" + synchronizedRulePackage.rulePackage.systemName + "\" to a textual rule package");

        // Make sure it is a visual rule package
        if (!synchronizedRulePackage.rulePackage is IVisualRulePackageVO) {
            throw new Error('Can only create textual versions of visual rule packages');
        } else {

            // Remove the visual rule package from the edited objects
            model.ruleEditorModel.removeRulePackage(synchronizedRulePackage);

            // Convert the rule package
            synchronizedRulePackage.rulePackage = (synchronizedRulePackage.rulePackage as IVisualRulePackageVO).getTextualPackage();

            // Add it to the edited objects
            model.ruleEditorModel.addRulePackage(synchronizedRulePackage);

        }
    }

}
}
