/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetRulePackagesCommand.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;

/**
 * Get the rule packages
 *
 * @author Julien Nicoulaud
 */
public class GetRulePackagesCommand extends AbstractCommand {

    public function GetRulePackagesCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    override public function execute(event:CairngormEvent):void {
        delegate.responder = this;
        delegate.getRulePackages(model.ruleEngineName);
    }

    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    override public function result(event:Object):void {
        // Store the result in the model
        var rulePackages:ArrayCollection = ((event as ResultEvent).result as ArrayCollection);

        // Dispatch a notification to Kerneos
        debug("Loading/reloading the rule packages");

        // Clear the old packages list (tricky: only way to make sure a
        // CollectionEvent is dispatched for each entry)
        var numOldPackages:int = model.ruleManagerModel.rulePackages.length;
        for (var i:int = 0; i < numOldPackages; i++) {
            model.ruleManagerModel.rulePackages.removeItemAt(0);
        }

        // Re-fill the packages list with the new ones
        for each (var rulePackage:IRulePackageVO in rulePackages) {

            // Wrap the rule package in a SynchronizedRulePackage
            var newSynchronizedRulePackage:SynchronizedRulePackage =
                    new SynchronizedRulePackage(rulePackage, SynchronizedRulePackageState.SYNCHRONIZED);

            // Add it to the collection
            model.ruleManagerModel.rulePackages.addItem(newSynchronizedRulePackage);

            // Launch a test compilation
            dispatcher.dispatchEvent(
                    new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE,
                            newSynchronizedRulePackage)
            );
        }

        // Mark the pending "Save" operations as cancelled
        model.ruleManagerModel.pendingSaveOperations = 0;
    }

}
}
