/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: UpdateRulePackageMetadataCommand.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;

import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;

/**
 * Update the rule package metadata when it has been modified externally
 *
 * @author Julien Nicoulaud
 */
public class UpdateRulePackageMetadataCommand extends AbstractCommand {

    /**
     * The traited rule package
     */
    private var synchronizedRulePackage:SynchronizedRulePackage = null;


    public function UpdateRulePackageMetadataCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    override public function execute(event:CairngormEvent):void {
        // Retrieve the target object
        synchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;

        // Dispatch a notification to Kerneos
        debug("Updating \"" + synchronizedRulePackage.rulePackage.systemName + "\" metadata");

         delegate.responder = this;
        if (synchronizedRulePackage.rulePackage.previousSystemName != null) {
            delegate.getRulePackage(synchronizedRulePackage.rulePackage.previousSystemName,
                    model.ruleEngineName);
        } else {
            delegate.getRulePackage(synchronizedRulePackage.rulePackage.systemName,
                    model.ruleEngineName);
        }
    }

    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    override public function result(event:Object):void {
        // Retrieve the rule package
        var rulePackage:IRulePackageVO = (event as ResultEvent).result as IRulePackageVO;

        // Calculate the new state
        var newState:SynchronizedRulePackageState = synchronizedRulePackage.state;
        if (synchronizedRulePackage.state == SynchronizedRulePackageState.OBSOLETE) {
            newState = SynchronizedRulePackageState.SYNCHRONIZED;
        } else if (synchronizedRulePackage.state == SynchronizedRulePackageState.OUT_OF_SYNC) {
            newState = SynchronizedRulePackageState.MODIFIED;
        }

        // Update only the fields that can be modified by the rules manager
        // Dirty: set the state as "modified" while assigning values to prevent
        // it from being automatically and reach incoherent values
        synchronizedRulePackage.state = SynchronizedRulePackageState.MODIFIED;
        synchronizedRulePackage.rulePackage.description = rulePackage.description;
        synchronizedRulePackage.rulePackage.state = rulePackage.state;
        synchronizedRulePackage.state = newState;
    }

}
}
