/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleEditorController.as 4905 2009-08-13 13:35:13Z nicoulaj $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.notificationeditor.control {
import com.adobe.cairngorm.control.CairngormEventDispatcher;
import com.adobe.cairngorm.control.FrontController;

import org.ow2.jasmine.monitoring.eos.notificationeditor.command.*;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.EngineDashboardEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RuleManagerEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageSystemNameEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackagesEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.ServerFilePathEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;


/**
 * Bind a type of command to an action.
 *
 * @author durieuxp
 */
public class Controller extends FrontController {

    /**
     * Create an instance of the controller. Commands are initialized here.
     */
    public function Controller(dispatcher:CairngormEventDispatcher) {
        super(dispatcher);
    }

    /**
     * Add all of the commands to the pool of commands.
     */
    public function initialiseCommands():void {
        addCommand(EngineDashboardEvent.UPDATE_ENGINE_STATUS, UpdateEngineStatusCommand);
        addCommand(EngineDashboardEvent.START_ENGINE, StartEngineCommand);
        addCommand(EngineDashboardEvent.STOP_ENGINE, StopEngineCommand);
        addCommand(SynchronizedRulePackageEvent.CHECK_NAME_AVAILABILITY, CheckSystemNameAvailabilityCommand);
        addCommand(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE, CompileRulePackageCommand);
        addCommand(SynchronizedRulePackageEvent.CONVERT_TO_TEXTUAL_PACKAGE, ConvertToTextualPackageCommand);
        addCommand(SynchronizedRulePackageEvent.CREATE_TEXTUAL_VERSION, CreateTextualVersionCommand);
        addCommand(SynchronizedRulePackageEvent.DELETE_RULE_PACKAGE, DeleteRulePackageCommand);
        addCommand(RulePackagesEvent.EXPORT_RULE_PACKAGES_AS_XML, ExportRulePackagesAsXMLCommand);
        addCommand(RulePackageSystemNameEvent.GET_RULE_PACKAGE, GetRulePackageCommand);
        addCommand(RuleManagerEvent.GET_RULE_PACKAGES, GetRulePackagesCommand);
        addCommand(ServerFilePathEvent.IMPORT_RULE_PACKAGE, ImportRulePackageCommand);
        addCommand(RulePackageSystemNameEvent.RELOAD_RULE_PACKAGE, ReloadRulePackageCommand);
        addCommand(SynchronizedRulePackageEvent.SAVE_AND_CLOSE_SYNCHRONIZED_RULE_PACKAGE, SaveRulePackageCommand);
        addCommand(SynchronizedRulePackageEvent.SAVE_SYNCHRONIZED_RULE_PACKAGE, SaveRulePackageCommand);
        addCommand(SynchronizedRulePackageEvent.UPDATE_RULE_PACKAGE_METADATA, UpdateRulePackageMetadataCommand);
    }

    public function removeCommands():void {
        removeCommand(EngineDashboardEvent.UPDATE_ENGINE_STATUS);
        removeCommand(EngineDashboardEvent.START_ENGINE);
        removeCommand(EngineDashboardEvent.STOP_ENGINE);
        removeCommand(SynchronizedRulePackageEvent.CHECK_NAME_AVAILABILITY);
        removeCommand(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE);
        removeCommand(SynchronizedRulePackageEvent.CONVERT_TO_TEXTUAL_PACKAGE);
        removeCommand(SynchronizedRulePackageEvent.CREATE_TEXTUAL_VERSION);
        removeCommand(SynchronizedRulePackageEvent.DELETE_RULE_PACKAGE);
        removeCommand(RulePackagesEvent.EXPORT_RULE_PACKAGES_AS_XML);
        removeCommand(RulePackageSystemNameEvent.GET_RULE_PACKAGE);
        removeCommand(RuleManagerEvent.GET_RULE_PACKAGES);
        removeCommand(ServerFilePathEvent.IMPORT_RULE_PACKAGE);
        removeCommand(RulePackageSystemNameEvent.RELOAD_RULE_PACKAGE);
        removeCommand(SynchronizedRulePackageEvent.SAVE_AND_CLOSE_SYNCHRONIZED_RULE_PACKAGE);
        removeCommand(SynchronizedRulePackageEvent.SAVE_SYNCHRONIZED_RULE_PACKAGE);
        removeCommand(SynchronizedRulePackageEvent.UPDATE_RULE_PACKAGE_METADATA);
    }
}
}
