/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleEditorModel.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain {

import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.EventDispatcher;

import mx.collections.ArrayCollection;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.EditorListChangeEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageListChangeEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.textual.TextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.VisualRulePackageVO;
import org.ow2.kerneos.common.event.KerneosNotificationEvent;

/**
 * Cairngorm Model for the rule editor
 *
 * @author Julien Nicoulaud
 */
[Bindable]
public class RuleEditorModel extends EventDispatcher {

    private const ruleEngineName:String = NotificationEditor.getInstance().getModel().ruleEngineName;

    /**
     * The visual edition mode state.
     */
    public var allowVisualEdition:Boolean = true;

    private const dispatcher:CairngormEventDispatcher = NotificationEditor.getInstance().getDispatcher();

    /**
     * The edited synchronized rule packages
     */
    [ArrayElementType("org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage")]
    private var editedRulePackages:ArrayCollection = new ArrayCollection;

    /**
     * The package containing the visual element types
     */
    public static var visualElementTypesPackage:String
            = 'org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.';

    /**
     * The allowed element types in visual edition mode
     */
    public var allowedVisualElementTypes:ArrayCollection
            = new ArrayCollection(['RHSCodeSnippetVO',
        'LHSCodeSnippetVO',
        'LogicalExpressionVO']);


    public function RuleEditorModel() {
        super();
    }

// =========================================================================
    // Public methods
    // =========================================================================

    /**
     * Create a "ready to be edited" textual rule package object and add it
     * to the edited objects
     */
    public function editNewTextualRulePackage():void {
        debug("editNewTextualRulePackage")
        var newRulePackage:IRulePackageVO = createNewTextualRulePackage();
        addRulePackage(new SynchronizedRulePackage(newRulePackage, SynchronizedRulePackageState.SYNCHRONIZED));
    }

    /**
     * Create a "ready to be edited" visual rule package object and add it
     * to the edited objects
     */
    public function editNewVisualRulePackage():void {
        debug("editNewVisualRulePackage")
        var newRulePackage:IRulePackageVO = createNewVisualRulePackage();
        addRulePackage(new SynchronizedRulePackage(newRulePackage, SynchronizedRulePackageState.SYNCHRONIZED));
    }

    /**
     * Create a new textual rule package
     */
    public function createNewTextualRulePackage():TextualRulePackageVO {
        debug("createNewTextualRulePackage")
        var date:Date = new Date();
        var newRulePackage:TextualRulePackageVO = new TextualRulePackageVO();
        newRulePackage.displayName = "New rule package (" + date.getDate()
                + "/" + (date.getMonth() + 1)
                + "/" + date.getFullYear()
                + " - " + date.getHours()
                + ":" + date.getMinutes()
                + ":" + date.getSeconds() + ")";
        newRulePackage.ruleEngineInstanceName = ruleEngineName;
        newRulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        newRulePackage.description =
                "Here is a rule package to start." +
                        "\nEnjoy !";
        newRulePackage.content =
                "package org.ow2.jasmine.rules.packages.NewEmptyRulePackage" +
                        "\n" +
                        "\nrule \"Empty rule\"" +
                        "\n    no-loop false" +
                        "\n    when" +
                        "\n        // Let's check some facts !" +
                        "\n    then" +
                        "\n        // Let's act !" +
                        "\nend\n";
        return newRulePackage;
    }

    /**
     * Create a new visual rule package
     */
    public function createNewVisualRulePackage():VisualRulePackageVO {
        debug("createNewVisualRulePackage")
        var date:Date = new Date();
        var newRulePackage:VisualRulePackageVO = new VisualRulePackageVO();
        newRulePackage.displayName = "New rule package (" + date.getDate()
                + "/" + (date.getMonth() + 1)
                + "/" + date.getFullYear()
                + " - " + date.getHours()
                + ":" + date.getMinutes()
                + ":" + date.getSeconds() + ")";
        newRulePackage.ruleEngineInstanceName = ruleEngineName;
        newRulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        newRulePackage.description =
                "Enter your description here !";
        //var newRule : IRuleVO = new RuleVO();
        //newRule.imports = newRulePackage.imports;
        //newRule.name = "My first rule";
        /*var newLHSElement : LogicalExpressionVO = new LogicalExpressionVO();
         newRule.lhs.addItem(newLHSElement);*/
        //var newLHSElement : LHSCodeSnippetVO = new LHSCodeSnippetVO();
        //newLHSElement.content = "// Let's check some facts !";
        //newLHSElement.imports = newRulePackage.imports;
        //newRule.lhs.addItem(newLHSElement);
        //var newRHSElement : RHSCodeSnippetVO = new RHSCodeSnippetVO();
        //newRHSElement.content = "// Let's act !";
        //newRHSElement.imports = newRulePackage.imports;
        //newRule.rhs.addItem(newRHSElement);
        //newRulePackage.rules.addItem(newRule);
        return newRulePackage;
    }

    /**
     * Test if a given rule package is already edited
     */
    public function getEditedRulePackage(rulePackageSystemName:String):SynchronizedRulePackage {
        debug("getEditedRulePackage")
        // Loop on the edited objects searching for this one
        for each (var editedRulePackage:SynchronizedRulePackage in editedRulePackages) {
            if (editedRulePackage.rulePackage.systemName == rulePackageSystemName) {
                return editedRulePackage;
            }
        }
        return null;
    }

    /**
     * Add a rule package to the edited objects
     */
    public function addRulePackage(rulePackage:SynchronizedRulePackage):void {
        debug("addRulePackage " + rulePackage);
        editedRulePackages.addItem(rulePackage);
        dispatchEvent(new RulePackageListChangeEvent(RulePackageListChangeEvent.ADD_RULE_PACKAGE_TYPE, rulePackage));
    }


    /**
     * Remove a rule package from the edited objects
     */
    public function removeRulePackage(rulePackage:SynchronizedRulePackage):void {
        debug("removeRulePackage " + rulePackage)
        editedRulePackages.removeItemAt(editedRulePackages.getItemIndex(rulePackage));
        dispatchEvent(new RulePackageListChangeEvent(RulePackageListChangeEvent.REMOVE_RULE_PACKAGE_TYPE, rulePackage));
    }

    protected function debug(mess:String):void {
        dispatcher.dispatchEvent(
                new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                        mess, KerneosNotificationEvent.DEBUG));
    }
}
}
