/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SynchronizedRulePackageState.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain {
/**
 * An enumerated type to represent the synch state of a
 * rule package value object
 *
 * @author Julien Nicoulaud
 */
[Bindable]
public final class SynchronizedRulePackageState {

    public function SynchronizedRulePackageState() {
    }

// =========================================================================
    // Utils
    // =========================================================================

    /**
     * The rule package is exactly the same on the client and on the server
     */
    public static const SYNCHRONIZED:SynchronizedRulePackageState
            = new SynchronizedRulePackageState();

    /**
     * The rule package was modified in the client
     */
    public static const MODIFIED:SynchronizedRulePackageState
            = new SynchronizedRulePackageState();

    /**
     * The client has an old of the rule package
     */
    public static const OBSOLETE:SynchronizedRulePackageState
            = new SynchronizedRulePackageState();

    /**
     * Both server and client modified the rule package, it needs
     * a merge.
     */
    public static const OUT_OF_SYNC:SynchronizedRulePackageState
            = new SynchronizedRulePackageState();


    // =========================================================================
    // Utils
    // =========================================================================

    /**
     * Formats the instance as a String
     */
    public function toString():String {
        switch (this) {
            case SYNCHRONIZED:
                return "Synchronised";
            case MODIFIED:
                return "Modified";
            case OBSOLETE:
                return "Obsolete";
            case OUT_OF_SYNC:
                return "Out of synchronisation";
        }
        return null;
    }
}
}
