/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleEditorMainViewPM.as 5621 2009-11-08 17:42:06Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.presentation {
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.EventDispatcher;

import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.EditorListChangeEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageListChangeEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackageSystemNameEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.ServerFilePathEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.utils.RulePackagesImportExportManager;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.textual.TextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.VisualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.RuleEditorModel;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.ISynchronizedRulePackageEditor;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.textual.TextualEditor;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.VisualEditor;
import org.ow2.kerneos.common.event.KerneosNotificationEvent;

/**
 * Presentation model for the rule editor
 *
 * @author Julien Nicoulaud
 */
[Bindable]
public class RuleEditorMainViewPM extends EventDispatcher {

    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * Domain model
     */
    public var ruleEditorModel:RuleEditorModel;

    /**
     * The rule packages editors
     */
    [ArrayElementType("org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.ISynchronizedRulePackageEditor")]
    private var ruleEditors:ArrayCollection = new ArrayCollection();

    private const dispatcher:CairngormEventDispatcher = NotificationEditor.getInstance().getDispatcher();

    // =========================================================================
    // Constructors
    // =========================================================================

    /**
     * Build the Presentation Model for a rule editor main view
     */
    public function RuleEditorMainViewPM(ruleEditorModel:RuleEditorModel) {
        // Set the model watched
        this.ruleEditorModel = ruleEditorModel;

        // Listen for model edited rules collection changes
        ruleEditorModel.addEventListener(RulePackageListChangeEvent.ADD_RULE_PACKAGE_TYPE, addEditor);
        ruleEditorModel.addEventListener(RulePackageListChangeEvent.REMOVE_RULE_PACKAGE_TYPE, removeEditor);
    }

    // =========================================================================
    // Model watching
    // =========================================================================

    private function addEditor(event:RulePackageListChangeEvent):void {
        debug("ADD editor")
        var editor:ISynchronizedRulePackageEditor = createNewEditor(event.rulepackage);
        ruleEditors.addItem(editor);
        var ev:EditorListChangeEvent = new EditorListChangeEvent(EditorListChangeEvent.ADD_ELEMENT_TYPE, editor);
        dispatchEvent(ev);
    }

    private function removeEditor(event:RulePackageListChangeEvent):void {
        debug("REMOVE editor")
        for each (var editor:ISynchronizedRulePackageEditor in ruleEditors) {
            if (editor.editedSynchronizedRulePackage == event.rulepackage) {
                ruleEditors.removeItemAt(ruleEditors.getItemIndex(editor));
            }
        }
        var ev:EditorListChangeEvent = new EditorListChangeEvent(EditorListChangeEvent.REMOVE_ELEMENT_TYPE, editor);
        dispatchEvent(ev);
    }

    // =========================================================================
    // Public methods
    // =========================================================================

    /**
     * Edit the given reference
     */
    public function editRulePackage(rulePackageSystemName:String = null):void {
        // If no rule package specified, create a new one
        if (rulePackageSystemName == null || rulePackageSystemName == "") {
            editNewRulePackage();
        } else if (ruleEditorModel.getEditedRulePackage(rulePackageSystemName) == null) {
            getRulePackage(rulePackageSystemName);
        } else {
            debug("editRulePackage ???")
        }
    }

    /**
     * Edit a new rule package
     */
    public function editNewRulePackage():void {
        // Edit in visual mode by default if visual mode enabled
        if (ruleEditorModel.allowVisualEdition) {
            editNewVisualRulePackage();
        }
        else {
            editNewTextualRulePackage();
        }
    }

    /**
     * Edit a new textual rule package
     */
    public function editNewTextualRulePackage():void {
        ruleEditorModel.editNewTextualRulePackage();
    }

    /**
     * Edit a new visual rule package
     */
    public function editNewVisualRulePackage():void {
        ruleEditorModel.editNewVisualRulePackage();
    }


    /**
     * Save an editor's rule package
     */
    public function saveEditor(targetEditor:ISynchronizedRulePackageEditor, closeAfterSuccessfulSave:Boolean = false):void {
        // Get the edited object and save it
        saveRulePackage(targetEditor.editedSynchronizedRulePackage,
                closeAfterSuccessfulSave);
    }

    /**
     * Close an editor without saving its contents
     */
    public function closeEditorWithoutSaving(targetEditor:ISynchronizedRulePackageEditor):void {
        // Get the edited object and close it
        closeRulePackage(targetEditor.editedSynchronizedRulePackage);
    }

    /**
     * Locate the editor for a file
     */
    public function locateEditor(rulePackageSystemName:String):ISynchronizedRulePackageEditor {
        // Loop on all the editors... lame.
        for each (var editor:ISynchronizedRulePackageEditor in ruleEditors) {
            if ((editor.editedSynchronizedRulePackage.rulePackage.previousSystemName != null &&
                    editor.editedSynchronizedRulePackage.rulePackage.previousSystemName == rulePackageSystemName) ||
                    editor.editedSynchronizedRulePackage.rulePackage.systemName == rulePackageSystemName) {
                return editor;
            }
            break;
        }
        return null;
    }

    /**
     * Export a rule package
     */
    public function exportRulePackages(rulePackages:Array):void {
        var exportManager:RulePackagesImportExportManager =
                new RulePackagesImportExportManager();
        exportManager.exportPackages(rulePackages);
    }

    /**
     * Import rule packages
     */
    public function importRulePackages():void {
        var importManager:RulePackagesImportExportManager =
                new RulePackagesImportExportManager();
        importManager.importRulePackages();
        importManager.addEventListener(LoadEvent.LOADED, fileToImportLoaded);
    }


    // =========================================================================
    // Private methods
    // =========================================================================

    /**
     * Create a new editor editing the given object
     */
    public function createNewEditor(srp:SynchronizedRulePackage):ISynchronizedRulePackageEditor {
        // Declare a new editor
        var editor:ISynchronizedRulePackageEditor;

        // Instanciate an editor depending on the edited object type
        if (srp.rulePackage is TextualRulePackageVO) {
            editor = new TextualEditor();
        } else if (srp.rulePackage is VisualRulePackageVO) {
            editor = new VisualEditor();
        } else {
            throw new Error('No editor could be created for this kind of object.');
        }

        editor.editedSynchronizedRulePackage = srp;
        return editor;
    }

    /**
     * Load a rule package identified by the systemName field
     */
    public function getRulePackage(rulePackageSystemName:String):void {
        new RulePackageSystemNameEvent(
                RulePackageSystemNameEvent.GET_RULE_PACKAGE,
                rulePackageSystemName
        ).dispatch(dispatcher);
    }

    /**
     * Save a rule package
     */
    private function saveRulePackage(rulePackage:SynchronizedRulePackage, closeAfterSuccessfulSave:Boolean = false):void {
        if (closeAfterSuccessfulSave) {
            new SynchronizedRulePackageEvent(
                    SynchronizedRulePackageEvent.SAVE_AND_CLOSE_SYNCHRONIZED_RULE_PACKAGE,
                    rulePackage
            ).dispatch(dispatcher);
        } else {
            new SynchronizedRulePackageEvent(
                    SynchronizedRulePackageEvent.SAVE_SYNCHRONIZED_RULE_PACKAGE,
                    rulePackage
            ).dispatch(dispatcher);
        }
    }

    /**
     * Close a rule package
     */
    private function closeRulePackage(rulePackage:SynchronizedRulePackage):void {
        ruleEditorModel.removeRulePackage(rulePackage);
    }

    /**
     * When a file to import as been uploaded
     */
    private function fileToImportLoaded(event:LoadEvent):void {
        dispatcher.dispatchEvent(
                new ServerFilePathEvent(ServerFilePathEvent.IMPORT_RULE_PACKAGE, event.filename)
        );
    }

    protected function debug(mess:String):void {
        dispatcher.dispatchEvent(
                new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                        mess, KerneosNotificationEvent.DEBUG));
    }
}
}
