/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.presentation.editor.visual
{
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.Event;
import flash.sampler.*;

import mx.binding.utils.*;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.NotificationEditorModelLocator;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.textual.ITextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.textual.TextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IVisualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.RuleEditorModel;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.presentation.editor.AbstractEditorPM;
    
/**
* Presentation model for the visual rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class VisualEditorPM extends AbstractEditorPM
{

    private const dispatcher:CairngormEventDispatcher = NotificationEditor.getInstance().getDispatcher();

    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The corresponding textual rule package
    */
    public var textualRP : ITextualRulePackageVO = new TextualRulePackageVO();
    
    
    // =========================================================================
    // Constructor
    // =========================================================================
    
    /**
    * Build the Presentation Model for an engine dashboard view
    */
    public function VisualEditorPM(editedSynchronizedRulePackage:SynchronizedRulePackage)
    {
        // Call super class constructor
        super(editedSynchronizedRulePackage);
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Refresh the source editor contents
    */
    public function refreshTextualRP():void
    {
        if (editedSynchronizedRulePackage != null) {
            textualRP = (editedSynchronizedRulePackage.rulePackage as IVisualRulePackageVO).getTextualPackage();
        }
    }
    
    /**
    * Convert the rule package to a textual one
    */
    public function switchToTextualMode(e:Event=null):void
    {
        // Retrieve the component model
        var model:RuleEditorModel = 
            NotificationEditor.getInstance().getModel().ruleEditorModel;
            
        // Dispatch the command event
        new SynchronizedRulePackageEvent(
            SynchronizedRulePackageEvent.CONVERT_TO_TEXTUAL_PACKAGE,
            editedSynchronizedRulePackage
        ).dispatch(dispatcher);
    }
    
    /**
    * Make a textual version of the rule package
    */
    public function duplicateAndOpenInTextualMode(e:Event=null):void
    {
        // Retrieve the component model
        var model:RuleEditorModel =
            NotificationEditor.getInstance().getModel().ruleEditorModel;
            
        // Dispatch the command event
        new SynchronizedRulePackageEvent(
            SynchronizedRulePackageEvent.CREATE_TEXTUAL_VERSION,
            editedSynchronizedRulePackage
        ).dispatch(dispatcher);
    }
    
    /**
    * Directly get the edited rule package
    * @internal Mandatory to allow data binding
    */
    public function get editedVisualRulePackage():IVisualRulePackageVO
    {
        return editedSynchronizedRulePackage.rulePackage as IVisualRulePackageVO;
    }
}
}
