/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.presentation.editor.visual
{
import flash.events.Event;
import flash.events.EventDispatcher;
import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IRuleVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IVisualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.RuleVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.IRuleEditor;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.RuleEditor;
    
/**
* Presentation model for the visual rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class VisualRulePackageEditorPM extends EventDispatcher
{
    
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The edited rule package
    */
    public var editedVisualRulePackage : IVisualRulePackageVO;
    
    /**
    * The rules editors
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.IRuleEditor')]
    public var ruleEditors : ArrayCollection = new ArrayCollection();
    
    
    // =========================================================================
    // Constructor
    // =========================================================================
    
    /**
    * Build the Presentation Model for an engine dashboard view
    */
    public function VisualRulePackageEditorPM(editedVisualRulePackage:IVisualRulePackageVO)
    {
        // Initialize variables
        this.editedVisualRulePackage = editedVisualRulePackage;
        
        // Initialize the editors collection
        initializeRuleEditors();
        
        // Watch the list of rules changes
        editedVisualRulePackage.rules.addEventListener(CollectionEvent.COLLECTION_CHANGE,
                                                       rulesListChangeHandler);
    }
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Create a new Rule object and add it to the rules list
    */
    public function addRule():void
    {
        editedVisualRulePackage.rules.addItem(new RuleVO);
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Initialize the collection of editors based on the existing list of rules
    */
    private function initializeRuleEditors():void
    {
        ruleEditors.removeAll();
        for each (var rule : IRuleVO in editedVisualRulePackage.rules) {
            ruleEditors.addItem(createNewEditor(rule));
        }
    }
    
    /**
    * Create a new editor for a Rule object
    */
    private function createNewEditor(rule:IRuleVO):IRuleEditor
    {
        // Declare a new editor
        var newEditor : IRuleEditor;
        
        // Instanciate an editor
        newEditor = new RuleEditor();
        newEditor.editedRule = rule;
        
        // Listen to demands from the editor
        newEditor.addEventListener("deleteMe",deleteMeHandler);

        return newEditor;
    }
    
    
    // =========================================================================
    // Event handlers
    // =========================================================================
    
    /**
    * When the edited rules list changes
    */
    private function rulesListChangeHandler(event:CollectionEvent):void
    {
        switch(event.kind) {
            case CollectionEventKind.ADD:
               for each (var newRule : IRuleVO in event.items) {
                   ruleEditors.addItem(createNewEditor(newRule));
               }
               break;
            case CollectionEventKind.REMOVE:
                for each (var removedRule : IRuleVO in event.items) {
                    for each (var editor : IRuleEditor in ruleEditors) {
                        if (editor.editedRule == removedRule) {
                            ruleEditors.removeItemAt(ruleEditors.getItemIndex(editor));
                        }
                    }
                }
                break;
            default:
                break;
        }
    }
    
    /**
    * When a child asks for deletion
    */
    private function deleteMeHandler(event:Event):void
    {
        editedVisualRulePackage.rules.removeItemAt(editedVisualRulePackage.rules.getItemIndex(
            (event.target as IRuleEditor).editedRule)
        );
    }
    
}
}
