/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RulePackagesImportExportManager.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.utils
{

import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.display.DisplayObject;
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.events.HTTPStatusEvent;
import flash.events.IOErrorEvent;
import flash.events.ProgressEvent;
import flash.events.SecurityErrorEvent;
import flash.net.FileFilter;
import flash.net.FileReference;
import flash.net.URLLoader;
import flash.net.URLRequest;
import flash.net.URLRequestMethod;
import flash.net.URLVariables;
import flash.utils.escapeMultiByte;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.core.FlexGlobals;
import mx.core.mx_internal;
import mx.events.CloseEvent;
import mx.managers.PopUpManager;

import org.ow2.jasmine.monitoring.eos.common.controls.ProgressBox;
import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
import org.ow2.jasmine.monitoring.eos.common.view.LoadWindow;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.ExportTypeChoiceWindow;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.RulePackagesEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;

/**
* A utilitary class to handle rule packages import and export
* 
* @author Julien Nicoulaud
*/
[Event(name="uploadComplete", type="org.ow2.jasmine.monitoring.eos.common.events.LoadEvent")]
public class RulePackagesImportExportManager extends EventDispatcher
{
    
    // =========================================================================
    // Properties
    // =========================================================================
    
    /**
    * The "Export rule packages" servlet URL
    */
    private static const EXPORT_RULES_PACKAGES_URL:String = "./ExportRulePackagesServlet";
    
    /**
    * The "Download file" servlet URL
    */
    private static const DOWNLOAD_FILE_URL:String = "./DownloadFileServlet";
    
    /**
    * Progress bar
    */
    private var progress:ProgressBox;
    
    /**
    * The window where the user chooses the export type
    */
    private var exportTypeChoiceWindow:ExportTypeChoiceWindow = null;

    /**
    * FileReference to download the exported file
    */
    private static var fileRef:FileReference = new FileReference();
    
    /**
    * The request made to download the file after it's ready
    */
    private static var downloadRequest : URLRequest;

    private static const dispatcher:CairngormEventDispatcher = NotificationEditor.getInstance().getDispatcher();

    
    
    // Assets
        
    /**
    * Export icon (48 pixels)
    */
    [Embed(source='/../assets/export48.png')]
    [Bindable]
    private static var exportIcon48:Class;
    
    
    // =========================================================================
    // Constructor
    // =========================================================================
        
    /**
    * Build a new RulePackagesImportExportManager
    */
    public function RulePackagesImportExportManager()
    {
    }
    
    
    // =========================================================================
    // Export type choice window management
    // =========================================================================
    
    /**
    * Export the given IRulePackage objects
    */
    public function exportPackages(packages:Array):void
    {
        // If the window is not already created
        if (exportTypeChoiceWindow == null) {
            
            // Create a window to force the user choosing a strategy
            exportTypeChoiceWindow = new ExportTypeChoiceWindow();
            PopUpManager.addPopUp(exportTypeChoiceWindow,
                                  FlexGlobals.topLevelApplication as DisplayObject,
                                  true);
            PopUpManager.centerPopUp(exportTypeChoiceWindow);
            
            // Listen to the window events
            exportTypeChoiceWindow.addEventListener(CloseEvent.CLOSE,
                                                      exportTypeChoiceWindowCloseHandler,false,0,true);
            exportTypeChoiceWindow.addEventListener('exportAsSource',
                                                    function():void {
                                                        exportPackagesAs(packages,"source");
                                                    },
                                                    false,
                                                    0,
                                                    true);
            exportTypeChoiceWindow.addEventListener('exportAsXML',
                                                    function():void {
                                                        exportPackagesAs(packages,"XML");
                                                    },
                                                    false,
                                                    0,
                                                    true);
            exportTypeChoiceWindow.addEventListener('exportAsBundle',
                                                    function():void {
                                                        exportPackagesAs(packages,"bundle");
                                                    },
                                                    false,
                                                    0,
                                                    true);
        }
    }
    
    /**
    * Export the given IRulePackage objects to the given target type
    */
    private function exportPackagesAs(packages:Array, targetType:String):void
    {
        // If exporting as XML
        if (targetType == "XML") {
            dispatcher.dispatchEvent(
                new RulePackagesEvent(RulePackagesEvent.EXPORT_RULE_PACKAGES_AS_XML,
                                      new ArrayCollection(packages))
            );
        }
        
        // Else, use the servlet
        else {
            
            // Prepare the request
            var exportRequest:URLRequest = new URLRequest();
            
            // Serialize selected rule packages data to a XML document
            var packagesData : String = "<?xml version=\"1.0\"?><packages>";
            for each (var rulePackage:IRulePackageVO in packages) {
                packagesData += "<package>";
                packagesData += '<name><!\[CDATA\[' + rulePackage.systemName.substr(rulePackage.systemName.lastIndexOf(".")+1) + '\]\]></name>';
                packagesData += "<source><!\[CDATA\[" + escapeMultiByte(rulePackage.getDrlSource()) + "\]\]></source>";
                packagesData += "</package>";
            }
            packagesData += "</packages>";
            
            // Build the request data
            var requestData : URLVariables = new URLVariables("packages=" + packagesData +
                                                              "&output=" + targetType);
            
            // Setup the request
            exportRequest.data = requestData;
            exportRequest.url = EXPORT_RULES_PACKAGES_URL;
            exportRequest.method = URLRequestMethod.POST;
            
            // Launch the request and listen to events
            var urlLoader:URLLoader = new URLLoader();
            urlLoader.addEventListener(Event.COMPLETE, completeHandler);
            urlLoader.addEventListener(ProgressEvent.PROGRESS, progressHandler);
            urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
            urlLoader.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
            urlLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
            urlLoader.load(exportRequest);
            
            // Display a progress box
            progress = PopUpManager.createPopUp(FlexGlobals.topLevelApplication as DisplayObject, ProgressBox) as ProgressBox;
            PopUpManager.centerPopUp(progress);
        }
    }
    
    /**
    * When the user closes the choice window
    */
    private function exportTypeChoiceWindowCloseHandler(e:Event):void
    {
        // Destroy the window object
        exportTypeChoiceWindow = null;
    }
    
    
    // =========================================================================
    // Export servlet response handling
    // =========================================================================
        
    /**
    * When the export is complete
    */
    private function completeHandler(event:Event):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Retrieve the servlet response
        var loader:URLLoader = URLLoader(event.target);
        var response : XML = XML(loader.data.toString());
        var path:String = response.file.@path;
        var name:String = response.file.@name;
        var size:String = response.file.@size;
        
        // Ask the user to download the generated file
        promptDownloadFile(path,name,size);
    }
    
    /**
    * Handles export progress events
    */
    private function progressHandler(event:ProgressEvent):void
    {
        // Update the progress bar informations
        progress.progressBar.setProgress(event.bytesLoaded, event.bytesTotal);
        progress.progressBar.label = (event.bytesLoaded / event.bytesTotal * 100).toFixed() + "%";
    }
    
    /**
    * Handles export security error events
    */
    private function securityErrorHandler(event:SecurityErrorEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        Alert.show("securityErrorHandler: " + event.toString());
    }
    
    /**
    * Handles HTTP status change events
    */    
    private function httpStatusHandler(event:HTTPStatusEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        // Alert.show("httpStatusHandler: " + event.toString());
    }
    
    /**
    * Handles export I/O error events
    */ 
    private function ioErrorHandler(event:IOErrorEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        Alert.show("ioErrorHandler: " + event.toString());
    }
    
    
    // =========================================================================
    // Result file download
    // =========================================================================
    
    /**
    * 
    */
    public static function promptDownloadFile(path:String,
                                              name:String=null,
                                              size:String=null):void {
                                          
        // Setup the "download file" request
        downloadRequest = new URLRequest(DOWNLOAD_FILE_URL + "?fileId=" + path);
        
        // Ask the user for its permission to download the file
        var readyToDownloadMessage : Alert =
            Alert.show('Do you want to save the file ' + ((name!=null)?name:"") + ((size!=null)?' (' + size + 'kb) ':' ') + 'to your hard drive ?',
                       'The file is ready to be downloaded',
                       (Alert.YES | Alert.NO),
                       null,
                       function(event:CloseEvent):void
                       {
                           if (event.detail == Alert.YES) {
                               fileRef.download(downloadRequest);
                           }
                       },
                       exportIcon48,
                       Alert.OK);
                       
        // A hack to use htmlText with an Alert
        readyToDownloadMessage.mx_internal::alertForm.mx_internal::textField.htmlText =
            "Do you want to save the file <i>" + name + "</i> (" + size + "kb) to your hard drive ?";
        
    }
    
    
    // =========================================================================
    // Rule packages import
    // =========================================================================
    
    /**
    * Display an interface to import rule packages
    */
    public function importRulePackages():void
    {
        var fileUploadWindow:LoadWindow = PopUpManager.createPopUp(FlexGlobals.topLevelApplication as DisplayObject,
                                                                   LoadWindow,
                                                                   true) as LoadWindow;
        var xmlTypes:FileFilter = new FileFilter("XML Rule packages (*.xml)", "*.xml");
        fileUploadWindow.fileFilter = new Array(xmlTypes);
        fileUploadWindow.title = "Import XML rule packages";
        fileUploadWindow.helpText = "Select one or several rule packages and upload them. " + 
                                    "If valid, they will be opened in the editor and you will be " + 
                                    "allowed to edit them and save them to the database.";
        fileUploadWindow.addEventListener(LoadEvent.LOADED, dispatchLoadEvent);
        PopUpManager.centerPopUp(fileUploadWindow);
    }
    
    /**
    * Dispatch an event informing the asking component that the file was uploaded
    */
    private function dispatchLoadEvent(event:Event):void
    {
        if (event is LoadEvent) {
            var loadEvent : LoadEvent = new LoadEvent(LoadEvent.LOADED);
            loadEvent.filename = (event as LoadEvent).filename;
            dispatchEvent(loadEvent);
        }
    }
}
}
