/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DroolsCodeArea.as 8512 2011-07-12 07:00:31Z diorcety $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor
{
import flash.events.Event;
import flash.events.FocusEvent;
import flash.events.MouseEvent;
import flash.text.TextFormat;

import mx.collections.ArrayCollection;
import mx.controls.TextArea;
import mx.controls.textClasses.TextRange;
import mx.core.mx_internal;
import mx.events.FlexEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.CodeArea;

//use namespace mx_internal;

/**
* A TextArea for Drools rule packages edition. Provides syntax highlighting.
*
* @author Julien Nicoulaud
* @see org.ow2.jasmine.monitoring.eos.common.controls.CodeArea
* @see org.ow2.jasmine.monitoring.eos.common.controls.LineNumberedTextArea
*/
public class DroolsCodeArea extends CodeArea
{
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * Syntax highlighting keywords groups
    */
    private var droolsTopLevelKeywords:ArrayCollection =
            new ArrayCollection(["package",
                                 "rule",
                                 "query",
                                 "when",
                                 "then",
                                 "end",
                                 "function",
                                 "global",
                                 "import",
                                 "template",
                                 "declare"]);
    private var droolsOperators:ArrayCollection =
            new ArrayCollection(["exists",
                                 "collect",
                                 "accumulate",
                                 "from",
                                 "forall",
                                 "eval",
                                 "and",
                                 "or",
                                 "not",
                                 "memberOf",
                                 "matches",
                                 "contains",
                                 "soundslike"]);
    private var droolsValueKeywords:ArrayCollection =
            new ArrayCollection(["null",
                                 "true",
                                 "false"]);
    private var droolsOptionsKeywords:ArrayCollection =
            new ArrayCollection(["enabled",
                                 "salience",
                                 "duration",
                                 "date-effective",
                                 "date-expires",
                                 "no-loop",
                                 "lock-on-active",
                                 "auto-focus",
                                 "activation-group",
                                 "agenda-group",
                                 "dialect",
                                 "ruleflow-group"]);
                                 
    /**
    * Syntax highlighting predefined formats
    */
    private var topLevelKeywordsFormat:TextFormat;
    private var optionsKeywordsFormat:TextFormat;
    private var valueKeywordsFormat:TextFormat;
    private var operatorsKeywordsFormat:TextFormat;
    private var commentFormat:TextFormat;
    private var quoteFormat:TextFormat;                                 


    // =========================================================================
    // Constructor
    // =========================================================================
    
    /**
    * Build a new DroolsCodeArea
    */
	public function DroolsCodeArea()
	{
		// Call super class constructor
		super();
        
        // Define syntax highlighting format options
        topLevelKeywordsFormat = new TextFormat();
        topLevelKeywordsFormat.color = 0xCC0000;
        topLevelKeywordsFormat.bold = true;
        topLevelKeywordsFormat.underline = false;

        optionsKeywordsFormat = new TextFormat();
        optionsKeywordsFormat.color = 0x3366CC;
        optionsKeywordsFormat.bold = true;
        optionsKeywordsFormat.underline = false;

        valueKeywordsFormat = new TextFormat();
        valueKeywordsFormat.color = 0x0000FF;
        valueKeywordsFormat.bold = true;
        valueKeywordsFormat.underline = false;

        operatorsKeywordsFormat = new TextFormat();
        operatorsKeywordsFormat.color = 0x3366CC;
        operatorsKeywordsFormat.bold = true;
        operatorsKeywordsFormat.underline = false;

        commentFormat = new TextFormat();
        commentFormat.color = 0x387000;
        commentFormat.underline = false;
        
        quoteFormat = new TextFormat();
        quoteFormat.color = 0x6F0F0F;
        quoteFormat.bold = true;
        quoteFormat.underline = false;
        
		// Listen to events
        //addEventListener(Event.ENTER_FRAME,highlightCode); // Too brutal !
        addEventListener(Event.CHANGE,highlightCode);
        addEventListener(FlexEvent.CREATION_COMPLETE,highlightCode);
        addEventListener(FlexEvent.UPDATE_COMPLETE,highlightCode);
        addEventListener(FocusEvent.FOCUS_IN,highlightCode);
        addEventListener(FlexEvent.SHOW,highlightCode);
        addEventListener(MouseEvent.MOUSE_UP,highlightCode);
        addEventListener(MouseEvent.MOUSE_WHEEL,highlightCode);
	}
    
    
    // =========================================================================
    // Syntax highlighting
    // =========================================================================
    
    /**
    * Highlight the Drools code
    */
    public function highlightCode(e:Event=null):void
    {
        try {
            
            // If the text is not empty
            if (text.length > 0) {
                
                // Reset all text formatting
                var resetRange:TextRange = new TextRange(this as TextArea);
                resetRange.color = 0x000000;
                resetRange.fontWeight = "normal";
                resetRange.textDecoration = "normal";

                // Loop on each line                
                var maxLine:int = (this as TextArea).mx_internal::getTextField().numLines;                   
                var isInQuote:Boolean = false;
                var isInComment:Boolean = false;
                var quoteStart:int = 0;
                var commentStart:int = 0;
                for (var lineIndex:int = 0;lineIndex<maxLine;lineIndex++) {
                    
                    // Loop on each character on the line
                    var lineText:String = (this as TextArea).mx_internal::getTextField().getLineText(lineIndex);
                    var lineOffset:uint = (this as TextArea).mx_internal::getTextField().getLineOffset(lineIndex);
                    var lineLength:uint = (this as TextArea).mx_internal::getTextField().getLineLength(lineIndex);
                    var pos:uint = lineOffset;
                    var word:String = "";
                    
                    for (var i:uint = 0;i<lineText.length;i++) {
                        var char:String = lineText.charAt(i);
                        pos = lineOffset + i;
                        
                        // If the current character is a separator
                        if (isSeparator(char)) {
                            
                            // If we are in a multi line comment
                            if (isInComment) {
                                
                                // If this marks the end of the comment
                                if (char == "/") {
                                    var previousChar:String = lineText.charAt(i-1);
                                    if (previousChar == "*") {
                                        applyFormat(commentFormat,commentStart,pos+1);
                                        isInComment = false;
                                    }
                                }
                                
                            // If we are not in a multi line comment
                            } else {
                                
                                // Check if the character marks the beggining of a ...
                                if (char == "/" && !isInQuote) {
                                    var nextChar:String = lineText.charAt(i+1);
                                    
                                    // ... multi line comment
                                    if (nextChar == "*") {
                                        commentStart = pos;
                                        isInComment = true;
                                    }
                                    
                                    // ... single line comment
                                    else if (nextChar == "/") {
                                        applyFormat(commentFormat, pos,(lineOffset + lineLength));
                                        break;
                                    }
                                    
                                } else {
                                
                                    // Check if the character marks the beggining or the end of a string
                                    if (char == "\"") {
                                        if (isInQuote) {
                                            applyFormat(quoteFormat,quoteStart,pos+1);
                                            isInQuote = false;
                                        } else {
                                            quoteStart = pos;
                                            isInQuote = true;
                                        }
                                    }
                                    
                                    // Check if the word was a reserved word
                                    else if (word.length != 0 && !isInQuote) {
                                        highlightWord(word,(pos-word.length),pos);
                                    }
                                }
                            }
                            
                            // Start a new word
                            word = "";
                        }
                                                    
                        // If the character is not a separator
                        else {
                            
                            // Keep building the word
                            word += char;
                        }
                    }
                }
                
                // Special case: the last word of the last line
                highlightWord(word,(pos-word.length),pos+1);
            }
        } catch (error:Error ) {
            // throw error;
            // Do nothing, this is just syntax highlighting
        }
    }

    /**
     * Checks whether the given string is a separator between two tokens
     */
    private function isSeparator(char:String):Boolean
    { 
        var numValid:String = "0123456789";
        var alphaValid:String = "abcdefghijklmnopqrstuvwxyz";
        
        if (alphaValid.indexOf(char.toLowerCase()) >= 0 ||
            numValid.indexOf(char.toLowerCase()) >= 0 ||
            char == "-") {
            return false;
        }
        return true;
    }
    
    /**
    * Match the given word against keywords groups, and apply some specific format
    * if found.
    */
    private function highlightWord(word:String,beginIndex:Number,endIndex:Number):void
    {
        var selectedFormat:TextFormat = null;
        
        // Match the word against keywords groups
        if (droolsTopLevelKeywords.contains(word.toLowerCase())) {
            selectedFormat = topLevelKeywordsFormat;
        } else if (droolsOptionsKeywords.contains(word.toLowerCase())) {
            selectedFormat = optionsKeywordsFormat;
        } else if (droolsOperators.contains(word.toLowerCase())) {
            selectedFormat = operatorsKeywordsFormat;
        } else if (droolsValueKeywords.contains(word.toLowerCase())) {
            selectedFormat = valueKeywordsFormat;
        }
        
        // Apply the selected format
        applyFormat(selectedFormat,beginIndex,endIndex);
    }
    
    /**
    * Apply the given style format to the given text range
    */
    private function applyFormat(format:TextFormat,beginIndex:Number,endIndex:Number):void
    {
        if (format!=null) {
            (this as TextArea).mx_internal::getTextField().setTextFormat(format,beginIndex,endIndex);
        }
    }
}
}
